/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.wps.v_1_0_0.sample.rips;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.MessageFormat;

import net.disy.ogc.wps.v_1_0_0.util.UnreachableCodeReachedException;
import net.disy.ogc.wpspd.v_1_0_0.Link;
import net.disy.ogc.wpspd.v_1_0_0.LinkType;
import net.disy.ogc.wpspd.v_1_0_0.Message;
import net.disy.ogc.wpspd.v_1_0_0.MessageType;
import net.disy.ogc.wpspd.v_1_0_0.ObjectFactory;
import net.disy.ogc.wpspd.v_1_0_0.WpspdUtils;

import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.Point;

public abstract class AbstractRipsProcessToWpsAdapter {
  protected static Log logger = LogFactory.getLog(AbstractRipsProcessToWpsAdapter.class);

  private static final String PARAMETER_COORDINATES = "coord"; //$NON-NLS-1$
  private static final String PARAMETER_TYPE = "type"; //$NON-NLS-1$
  private static final String PARAMETER_ID = "id"; //$NON-NLS-1$
  private static final String PARAMETER_REQUEST = "request"; //$NON-NLS-1$
  private static final String REQUEST_TYPE_EXECUTE = "execute"; //$NON-NLS-1$
  private static String RESULT_ELEMENT_NAME = "result"; //$NON-NLS-1$
  private static String TEXT_ELEMENT_NAME = "text"; //$NON-NLS-1$
  private static String ERROR_ELEMENT_NAME = "error"; //$NON-NLS-1$

  protected Link executeWithLinkResult(String serviceId, Geometry geometry) {
    String baseUrl = getBaseUrl();
    String type = getType(geometry);
    String coordinateString = getCoordinateString(geometry);
    String url = MessageFormat.format("{0}?{1}={2}&{3}={4}&{5}={6}&{7}={8}", //$NON-NLS-1$
        baseUrl,
        PARAMETER_REQUEST,
        REQUEST_TYPE_EXECUTE,
        PARAMETER_ID,
        serviceId,
        PARAMETER_TYPE,
        type,
        PARAMETER_COORDINATES,
        coordinateString);
    logger.debug("URL: " + url);
    ObjectFactory objectFactory = WpspdUtils.createObjectFactory();
    LinkType link = objectFactory.createLinkType();
    link.setHref(url);
    link.setShow("new"); //$NON-NLS-1$
    return objectFactory.createLink(link);
  }

  //  protected Message executeWithTextResult(String serviceId, Geometry geometry) {
  //    HttpClient httpClient = new HttpClient();
  //    HttpMethod method = createGetMethod(serviceId, geometry);
  //    try {
  //      logger.debug("URI: " + method.getURI().toString());
  //    }
  //    catch (URIException e) {
  //      logger.debug("uri cant be resolved", e);
  //    }
  //    try {
  //      httpClient.executeMethod(method);
  //      InputStream inputStream = method.getResponseBodyAsStream();
  //      return createMessage(inputStream);
  //    }
  //    catch (HttpException e) {
  //      throw new RuntimeException(e);
  //    }
  //    catch (IOException e) {
  //      throw new RuntimeException(e);
  //    }
  //  }

  private HttpMethod createGetMethod(String serviceId, Geometry geometry) {
    String baseUri = getBaseUrl();
    HttpMethod method = new GetMethod(baseUri);
    method.setQueryString(createRequestParameters(serviceId, geometry));
    return method;
  }

  private String getBaseUrl() {
    String pattern = "http://rips-dienste.lubw.baden-wuerttemberg.de" //$NON-NLS-1$
        + "/rips/ripsweb.webservices/RipsServicesProvider.aspx"; //$NON-NLS-1$
    return pattern;
  }

  private NameValuePair[] createRequestParameters(String serviceId, Geometry geometry) {
    String type = getType(geometry);
    String coordinateString = getCoordinateString(geometry);
    return new NameValuePair[]{
        new NameValuePair(PARAMETER_REQUEST, REQUEST_TYPE_EXECUTE),
        new NameValuePair(PARAMETER_ID, serviceId),
        new NameValuePair(PARAMETER_TYPE, type),
        new NameValuePair(PARAMETER_COORDINATES, coordinateString) };
  }

  private String getType(Geometry geometry) {
    if (geometry instanceof Point) {
      return "point"; //$NON-NLS-1$
    }

    if (geometry instanceof LineString) {
      return "polyline"; //$NON-NLS-1$
    }

    throw new IllegalArgumentException("Type '" //$NON-NLS-1$
        + geometry.getGeometryType()
        + "' is not supported."); //$NON-NLS-1$
  }

  private String getCoordinateString(Geometry geometry) {
    boolean first = true;
    StringBuffer coordinateBuffer = new StringBuffer();
    for (Coordinate coordinate : geometry.getCoordinates()) {
      if (!first) {
        coordinateBuffer.append("|"); //$NON-NLS-1$
      }

      coordinateBuffer.append(coordinate.x + ";" + coordinate.y); //$NON-NLS-1$

      first = false;
    }
    try {
      return URLEncoder.encode(coordinateBuffer.toString(), "UTF-8");
    }
    catch (UnsupportedEncodingException e) {
      throw new UnreachableCodeReachedException();
    }
  }

  //  private Message createMessage(InputStream inputStream) throws IOException {
  //    final Element element = DocumentUtilities.read(inputStream).getRootElement();
  //    if (RESULT_ELEMENT_NAME.equals(element.getName())) {
  //
  //      final Element contentElement = ElementUtilities.getSingleRequiredElement(element);
  //
  //      if (TEXT_ELEMENT_NAME.equals(contentElement.getName())) {
  //        return createMessage(contentElement.getText(), "info"); //$NON-NLS-1$
  //      }
  //      else if (ERROR_ELEMENT_NAME.equals(contentElement.getName())) {
  //        return createMessage(contentElement.getText(), "error"); //$NON-NLS-1$
  //      }
  //      else {
  //        throw new XmlParseException("Unexpected content element [" //$NON-NLS-1$
  //            + contentElement.getName()
  //            + "], expected either [" //$NON-NLS-1$
  //            + TEXT_ELEMENT_NAME
  //            + "] or [" //$NON-NLS-1$
  //            + ERROR_ELEMENT_NAME
  //            + "]"); //$NON-NLS-1$
  //
  //      }
  //    }
  //    throw new XmlParseException("Unexpected root element [" + element.getName() + "]."); //$NON-NLS-1$ //$NON-NLS-2$
  //  }

  private Message createMessage(String content, String type) {
    ObjectFactory objectFactory = WpspdUtils.createObjectFactory();
    MessageType message = objectFactory.createMessageType();
    message.setContent(content);
    message.setType(type);
    return objectFactory.createMessage(message);
  }
}
